#include "windows.h"
#include "demo.h"
#include "ids.h"
#include "three_d.h"


HANDLE hInst;
static HWND hMainWindow;
static char szHeader[]="3-D Demo - By Ray Donahue";

extern BOOL FAR PASCAL DemoDlgProc(HWND,WORD,WORD,LONG);
extern BOOL FAR PASCAL ComboDlgProc(HWND,WORD,WORD,LONG);

/****************************************************************************

    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop

    COMMENTS:

        Windows recognizes this function by name as the initial entry point 
        for the program.  This function calls the application initialization 
        routine, if no other instance of the program is running, and always 
        calls the instance initialization routine.  It then executes a message 
        retrieval and dispatch loop that is the top-level control structure 
        for the remainder of execution.  The loop is terminated when a WM_QUIT 
        message is received, at which time this function exits the application 
        instance by returning the value passed by PostQuitMessage(). 

        If this function must abort before entering the message loop, it 
        returns the conventional value NULL.  

****************************************************************************/

int PASCAL WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;            /* current instance         */
HANDLE hPrevInstance;                      /* previous instance        */
LPSTR lpCmdLine;             /* command line             */
int nCmdShow;                /* show-window type (open/icon) */
{
    MSG msg;                 /* message                         */

    if (!hPrevInstance)                /* Other instances of app running? */
   if (!InitApplication(hInstance)) /* Initialize shared things */
       return (FALSE);          /* Exits if unable to initialize     */

    /* Perform initializations that apply to a specific instance */

    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    /* Acquire and dispatch messages until a WM_QUIT message is received. */

    while (GetMessage(&msg,       /* message structure                        */
       NULL,         /* handle of window receiving the message */
       NULL,         /* lowest message to examine               */
       NULL))        /* highest message to examine       */
   {
         TranslateMessage(&msg);        /* Translates virtual key codes      */
         DispatchMessage(&msg);         /* Dispatches message to window      */
    }
    return (msg.wParam);          /* Returns the value from PostQuitMessage */
}


/****************************************************************************

    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class

    COMMENTS:

        This function is called at initialization time only if no other 
        instances of the application are running.  This function performs 
        initialization tasks that can be done once for any number of running 
        instances.  

        In this case, we initialize a window class by filling out a data 
        structure of type WNDCLASS and calling the Windows RegisterClass() 
        function.  Since all instances of this application use the same window 
        class, we only need to do this when the first instance is initialized.  


****************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;              /* current instance       */
{
    WNDCLASS  wc;

    /* Fill in window class structure with parameters that describe the       */
    /* main window.                                                           */

    wc.style = CS_HREDRAW|CS_VREDRAW;                    /* Class style(s).                    */
    wc.lpfnWndProc = MainWndProc;       /* Function to retrieve messages for  */
                                        /* windows of this class.             */
    wc.cbClsExtra = 0;                  /* No per-class extra data.           */
    wc.cbWndExtra = 0;                  /* No per-window extra data.          */
    wc.hInstance = hInstance;           /* Application that owns the class.   */
    wc.hIcon = LoadIcon(hInstance, "ddd");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(LTGRAY_BRUSH); 
    wc.lpszMenuName =  "DemoMenu";   /* Name of menu resource in .RC file. */
    wc.lpszClassName = "DemoWClass"; /* Name used in call to CreateWindow. */

    /* Register the window class and return success/failure code. */

   return(RegisterClass(&wc));
}


/****************************************************************************

    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window

    COMMENTS:

        This function is called at initialization time for every instance of 
        this application.  This function performs initialization tasks that 
        cannot be shared by multiple instances.  

        In this case, we save the instance handle in a static variable and 
        create and display the main program window.  
        
****************************************************************************/

BOOL InitInstance(hInstance, nCmdShow)
    HANDLE          hInstance;          /* Current instance identifier.       */
    int             nCmdShow;           /* Param for first ShowWindow() call. */
{
    HWND            hWnd;               /* Main window handle.                */

    /* Save the instance handle in static variable, which will be used in  */
    /* many subsequence calls from this application to Windows.            */

    hInst = hInstance;

    /* Create a main window for this application instance.  */

    hWnd = CreateWindow(
        "DemoWClass",                /* See RegisterClass() call.          */
        "3-D Demonstration Program",   /* Text for window title bar.         */
        WS_OVERLAPPEDWINDOW,            /* Window style.                      */
        0,                  /* Default horizontal position.       */
        0,                  /* Default vertical position.         */
        GetSystemMetrics(SM_CXSCREEN),                  /* Default width.                     */
        GetSystemMetrics(SM_CYSCREEN),                  /* Default height.                    */
        NULL,                           /* Overlapped windows have no parent. */
        NULL,                           /* Use the window class menu.         */
        hInstance,                      /* This instance owns this window.    */
        NULL                            /* Pointer not needed.                */
    );

    /* If window could not be created, return "failure" */

    if (!hWnd)
        return (FALSE);

   hMainWindow = hWnd;

    /* Make the window visible; update its client area; and return "success" */
   ShowWindow(hMainWindow,nCmdShow);
   UpdateWindow(hMainWindow);

    return (TRUE);               /* Returns the value from PostQuitMessage */

}

/****************************************************************************
    FUNCTION: MainWndProc(HWND, unsigned, WORD, LONG)
****************************************************************************/

long FAR PASCAL MainWndProc(hWnd, message, wParam, lParam)
HWND hWnd;             /* window handle                  */
unsigned message;         /* type of message             */
WORD wParam;              /* additional information      */
LONG lParam;              /* additional information      */
{
   static FARPROC lpprocDemoDlg;
   static HFONT hFont;
   PAINTSTRUCT ps;
   RECT rc;
 
    switch (message) {

   case WM_CREATE:
      hFont = CreateFont(40,5,0,0,700,0,0,0,ANSI_CHARSET,OUT_DEFAULT_PRECIS,
       CLIP_DEFAULT_PRECIS,PROOF_QUALITY,VARIABLE_PITCH|FF_SWISS,"Helv");
      return(DefWindowProc(hWnd,message,wParam,lParam));

   case WM_DESTROY:
      DeleteObject(hFont);
      PostQuitMessage(0);
      break;

   case WM_PAINT:
      BeginPaint(hWnd,&ps);
      Draw3dFrame(hWnd,INSIDE_FRAME);
      SelectObject(ps.hdc,hFont);
      GetClientRect(hWnd,&rc);
      SetBkMode(ps.hdc,TRANSPARENT);
      SetTextColor(ps.hdc,GetSysColor(COLOR_BTNSHADOW));
      OffsetRect(&rc,2,2);
      DrawText(ps.hdc,(LPSTR)szHeader,lstrlen(szHeader),&rc,
         DT_SINGLELINE|DT_CENTER|DT_VCENTER);
      SetTextColor(ps.hdc,RGB(255,255,255));
      OffsetRect(&rc,-2,-2);
      DrawText(ps.hdc,(LPSTR)szHeader,lstrlen(szHeader),&rc,
         DT_SINGLELINE|DT_CENTER|DT_VCENTER);
      EndPaint(hWnd,&ps);
      break;

   case WM_COMMAND:     /* message: command from application menu */
      switch (wParam)
      {
         case IDM_DEMO:
            lpprocDemoDlg = MakeProcInstance(DemoDlgProc,hInst);
            DialogBox(hInst,"DemoBox",hWnd,lpprocDemoDlg);
            FreeProcInstance(lpprocDemoDlg);
            break;

         case IDM_COMBOLIST:
            lpprocDemoDlg = MakeProcInstance(ComboDlgProc,hInst);
            DialogBox(hInst,"ComboList",hWnd,lpprocDemoDlg);
            FreeProcInstance(lpprocDemoDlg);
            break;

         case IDM_README:
            WinExec("notepad 3d.txt",SW_SHOWNORMAL);
            break;

         default:
            break;
      }
      break;

   default:         /* Passes it on if unproccessed    */
       return (DefWindowProc(hWnd, message, wParam, lParam));
   }
   return (NULL);
}

