/* pulldown.c: Skeleton of menu bar with pull-down menus */

/* INCLUDES */
#include <graph.h>
#include <stdio.h>
#include <dos.h>
#include <conio.h>
#include <ctype.h>
#include "textscrn.h"
#include "popup.h"

/* DEFINES */
#if !defined TRUE
#define FALSE 0
#define TRUE !FALSE
#endif
#define CR 13
#define SPC 32
#define ESC 27
#define NONE 99

/* OBJECT DEFINITIONS */
POPUP full = {1, 1, 25, 80, 0, YELLOW, 0, BLACK};
POPUP action = {12, 25, 14, 55, 2, RED, 0, LTGRAY, 0};

/* Menu bar */
char menutext [] = "File\nPrint\nData\nGraph\nOptions";
MENUBAR bar = {1, 16, WHITE, BLUE, menutext};

/* Text of pull-down menus */
char    p0text [] = "Retrieve\nSave\nCombine\nBackup\nQuit",
		p1text [] = "Report\nGraph\nNew page",
		p2text [] = "Edit\nDelete\nInsert\nBrowse",
		p3text [] = "Type\nView\nDelete\nSave",
		p4text [] = "Colors\nKeyboard\nRecalc";

POPUP pop [] = {
	{3,  2, 7,  9, 1, BLACK, BLACK, LTGRAY, GREEN, p0text},
	{3, 16, 5, 23, 1, BLACK, BLACK, LTGRAY, GREEN, p1text},
	{3, 32, 6, 37, 1, BLACK, BLACK, LTGRAY, GREEN, p2text},
	{3, 48, 6, 53, 1, BLACK, BLACK, LTGRAY, GREEN, p3text},
	{3, 64, 5, 71, 1, BLACK, BLACK, LTGRAY, GREEN, p4text}
};

/* LOCAL FUNCTIONS */
void prompt (char*, POPUP*);
int doMenu (int, POPUP*);
void actOn (int, int);
/*--------------------------------------------------------------------*/

void main()
{
int repeating, pick;
char sel;
POPUP *popup;
struct videoconfig video;

/* Prepare main screen */
	_settextcolor (YELLOW);
	_setbkcolor (BLACK);
	_clearscreen (_GCLEARSCREEN);
	_getvideoconfig (&video);
	if (video.monitor == _MONO) {   /* change colors for mono */
		bar.fore = BLACK;
		bar.back = LTGRAY;
	}
	menubar (&bar);                 /* display menu bar */
	_settextposition (4, 1);
	_outtext ("Three of the pull-down menus overlay this text");

	/* Loop to drive menu bar selections */
	do {
		repeating = TRUE;                   /* set switch */
		prompt ("Pick menu selection by letter", &full);
		sel = getch();                  /* get selection */
		switch (toupper (sel)) {
			case 'F': pick = 0; break;
			case 'P': pick = 1; break;
			case 'D': pick = 2; break;
			case 'G': pick = 3; break;
			case 'O': pick = 4; break;
			default: pick = NONE; break;
		}
		if (pick < NONE) {
			popup = &(pop [pick]);          /* select pulldown */
			repeating = doMenu (pick, popup);   /* display it */
		}
	}   while (repeating);

	/* CLean up at end of job */
	_settextcolor (LTGRAY);
	_setbkcolor (BLACK);
	_clearscreen (_GCLEARSCREEN);
} /*-----------------------------------------------------*/


void prompt (char *message, POPUP *current)

			/* Display a prompt message at bottom of screen */
{
	popKeep (current);              /* save current state */
	popUse (&full);                 /* use full screen */
	_settextposition (24, 1);       /* go to prompt line */
	_cleareol();                        /* and clear it */
	popCenter (&full, 24, message);         /* display prompt */
	popUse (current);               /* return to calling popup */
} /*-----------------------------------------------------*/

int doMenu (int pick, POPUP *menu)

				/* Display pull-down menu, get selection */
{
	int indic = 1, keepOn = TRUE, looping = TRUE, last;

	_savescrn (0);                  /* save the display */
	prompt ("Space to advance, Enter to select, ESC to quit", menu);
	popShow (menu);             /* display the pulldown */
	popHilite (menu, indic);            /* highlight first choice */
	last = menu->bottom - menu->top + 1;    /* last menu row */
	do {
		switch (getch ()) {
			case SPC:                       /* advance */
				popNormal (menu, indic);        /* reset to normal */
				if (++indic > last)
					indic = 1;              /* wrap to top */
				popHilite (menu, indic);    /* hilite next */
				break;
			case ESC:                       /* quit */
				looping = FALSE;
				break;
			case CR:                        /* select */
				if ((pick == 0) && (indic == 5))
					keepOn = FALSE;             /* if picked "Quit" */
				else
					actOn (pick, indic);        /* act on choice */
				looping = FALSE;
				break;
			}
		} while (looping);
	if (!looping)
		_restscrn (0);      /* erase menu when done with it */
	return (keepOn);
} /*-----------------------------------------------------*/


void actOn (int barchoice, int menuchoice)

					/* stub of dispatcher for acting on menu choice */
{
	char report [30], key;

	_savescrn (0);                  /* save the display */
	sprintf (report, "You selected menu %d, item %d",
					barchoice, menuchoice);
	popShow (&action);
	popCenter (&action, 2, report);
	prompt ("Press any key to continue...", &action);
	getch();
	_restscrn (0);              /* restore after keypress */
} /*--------------------------------------------------------------*/

